# PAPI/SPOCP connection 
# Copyright (c) 2003-2005, RedIRIS. All Rights Reserved.
#
# You may distribute under the terms of the GNU General Public License,
# as specified in the LICENSE file that was shipped with this distribution.
#
# This module implements PAPI authorization through a SPOCP server
# (see http://www.spocp.org/ for details), although it can be used
# as an (almost) general interface from a Perl application to a SPOCP server.
#
package PAPI::SPOCP;
use strict;
use IO::Socket;
use IO::Socket::SSL;

sub new {
  my $class = shift;
  unshift(@_,'connectTo') if (@_ == 1);
  my %arg = @_;
  my $spocp = {};
  $spocp->{useTLS} = 0;
  $spocp->{peer} = undef;
  $spocp->{sock} = undef;
  $spocp->{to} = 5;
  $spocp->{proto} = 'tcp';
  $spocp->{peer} = $arg{connectTo} if (exists $arg{connectTo});
  $spocp->{to} = $arg{timeout} if (exists $arg{timeout});
  $spocp->{proto} = $arg{protocol} if (exists $arg{protocol});
  bless $spocp;
  $spocp->connect();
  if (defined $spocp->{sock}) { return $spocp; }
  else { return undef; }
}

sub connect {
  my $self = shift;
  if (!defined $self->{peer}) { return undef; }
  if ($self->{peer} =~ m/^\//) {
    $self->{sock} = new IO::Socket::UNIX ($self->{peer});
  }
  else {
    $self->{sock} = new IO::Socket::INET ('PeerAddr' => $self->{peer},
                                          'Timeout' => $self->{to},
                                          'Proto' => $self->{proto});
  }
  return $self->{sock};
}

sub setTLS {
  my $self = shift;
  if (!defined $self->{sock}) { return $self->{useTLS}; }
  my %arg = @_;
  my @opts = (SSL_use_cert => 1,        # Send client cert to server
              SSL_verify_mode => 0x01); # SSL_VERIFY_PEER
  if (exists $arg{certFile}) { push @opts, (SSL_cert_file => $arg{certFile}); }
  if (exists $arg{keyFile}) { push @opts, (SSL_key_file => $arg{keyFile}); }
  if (exists $arg{keyPassword}) {push @opts,(SSL_passwd_cb=>$arg{keyPassword});}
  if (exists $arg{caFile}) { push @opts, (SSL_ca_file => $arg{caFile}); }
  if (exists $arg{caDir}) { push @opts, (SSL_ca_path => $arg{caDir}); }
  if (exists $arg{serverDN}) { $self->{acceptDN} = $arg{serverDN}; }
  my $rt;
  $self->{sock}->send("10:8:STARTTLS\r\n");
  $self->{sock}->recv($rt,128);
  if ($rt !~ m/2:OK$/i) { return $self->{useTLS}; }
  my $nsock = IO::Socket::SSL::socket_to_SSL($self->{sock}, @opts);
  if (!defined $nsock) { return $self->{useTLS}; }
  if (defined $self->{acceptDN}) {
    my $peerDN = $nsock->peer_certificate('subject');
    if ($peerDN ne $self->{acceptDN}) {
      $self->{sock}->close(SSL_no_shutdown => 1);
      return $self->{useTLS};
    }
  }
  $self->{useTLS} = 1;
  return $self->{useTLS};
}

sub query {
  my $self = shift;
  if (!defined $self->{sock}) { return -1; }
  my $res = shift;
  my $act = shift;
  my $sub = shift;
  my $query = '(4:papi(3:poa'.
               $self->normalize($res).')(6:action'.
               $self->normalize($act).')(7:subject'.
               $self->normalizeSubject($sub).'))';
  my $rt;
  $query = $self->normalize("5:QUERY".$self->normalize($query));
  $self->{sock}->print("$query\r\n");
  if ($self->{useTLS}) { $rt = $self->{sock}->readline(); }
  else { $self->{sock}->recv($rt,512); }
  if ($rt =~ m/2:OK$/i) { return 1; }
  else { return 0; }
}

sub close {
  my $self = shift;
  $self->{sock}->send('LOGOUT\r\n');
  if ($self->{useTLS}) { $self->{sock}->close(SSL_no_shutdown => 1); }
  $self->{sock}->shutdown();
  $self->{sock} = undef;
}

sub normalize {
  my $self = shift;
  my $str = shift;
  return length($str).':'.$str;
}

sub normalizeSubject {
  my $self = shift;
  my $comr = shift;
  my @com = @{$comr};
  my $str = '';
  for (my $i = 0; $i < $#com; $i += 2) {
    $str .= '('.length($com[$i]).':'.$com[$i].
                length($com[$i+1]).':'.$com[$i+1].')';
  }
  return $str;
}

1;
